from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class ImagePullPolicyAlways(BaseK8Check):

    def __init__(self):
        """
        Image pull policy should be set to always to ensure you get the correct image and imagePullSecrets are correct
        Default is 'IfNotPresent' unless image tag is omitted or :latest
        https://kubernetes.io/docs/concepts/configuration/overview/#container-images

        An admission controller could be used to enforce imagePullPolicy
        """
        name = "Image Pull Policy should be Always"
        id = "CKV_K8S_15"
        supported_kind = ['containers', 'initContainers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return conf['parent'] + '.imagePullPolicy'

    def scan_spec_conf(self, conf):
        if "image" in conf:
            # Split on :
            if ":" in conf["image"]:
                (image, tag) = conf["image"].split(':')
            else:
                image = conf["image"]
                tag = ""
            if "imagePullPolicy" not in conf:
                if tag == "latest" or tag == "":
                    # Default imagePullPolicy = Always
                    return CheckResult.PASSED
                else:
                    # Default imagePullPolicy = IfNotPresent
                    return CheckResult.FAILED
            else:
                if conf["imagePullPolicy"] != "Always":
                    return CheckResult.FAILED
        else:
            return CheckResult.FAILED
        return CheckResult.PASSED

check = ImagePullPolicyAlways()


from abc import abstractmethod

from checkov.common.checks.base_check import BaseCheck
from checkov.kubernetes.registry import registry


class BaseK8Check(BaseCheck):
    def __init__(self, name, id, categories, supported_entities):
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities,
                         block_type="k8")
        self.supported_specs = supported_entities
        registry.register(self)

    @abstractmethod
    def scan_spec_conf(self, conf):
        raise NotImplementedError()

    def scan_entity_conf(self, conf):
        return self.scan_spec_conf(conf)

    @abstractmethod
    def get_resource_id(self, conf):
        pass



from abc import ABC, abstractmethod

from checkov.runner_filter import RunnerFilter


class BaseRunner(ABC):
    check_type = ""

    @abstractmethod
    def run(self, root_folder, external_checks_dir=None, files=None, runner_filter=RunnerFilter()):
        pass



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class SeccompPSP(BaseK8Check):

    def __init__(self):
        # CIS-1.5 5.7.2
        name = "Ensure default seccomp profile set to docker/default or runtime/default"
        id = "CKV_K8S_32"
        supported_kind = ['PodSecurityPolicy']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return 'PodSecurityPolicy.annotations.seccomp.security.alpha.kubernetes.io/defaultProfileName'

    def scan_spec_conf(self, conf):
        if "metadata" in conf:
            if "annotations" in conf["metadata"]:
                if "seccomp.security.alpha.kubernetes.io/defaultProfileName" in conf["metadata"]["annotations"]:
                    if ("docker/default" in conf["metadata"]["annotations"]["seccomp.security.alpha.kubernetes.io/defaultProfileName"] or
                        "runtime/default" in conf["metadata"]["annotations"]["seccomp.security.alpha.kubernetes.io/defaultProfileName"]):
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = SeccompPSP()




from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class DefaultServiceAccount(BaseK8Check):

    def __init__(self):
        # CIS-1.5 5.1.5
        name = "Ensure that default service accounts are not actively used"
        # Check automountServiceAccountToken in default service account in runtime
        id = "CKV_K8S_41"
        supported_kind = ['ServiceAccount']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return 'ServiceAccount.automountServiceAccountToken'

    def scan_spec_conf(self, conf):
        if "metadata" in conf:
            if "name" in conf["metadata"]:
                if conf["metadata"]["name"] == "default":
                    if "automountServiceAccountToken" in conf:
                        if conf["automountServiceAccountToken"] == False:
                            return CheckResult.PASSED
                    return CheckResult.FAILED
                else:
                    return CheckResult.PASSED
        return CheckResult.PASSED

check = DefaultServiceAccount()






from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class S3AccessLogs(BaseResourceCheck):
    def __init__(self):
        name = "Ensure the S3 bucket has access logging enabled"
        id = "CKV_AWS_18"
        supported_resources = ['AWS::S3::Bucket']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf['Properties'].get('LoggingConfiguration') is not None:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = S3AccessLogs()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck

class SecurityGroupRuleDescription(BaseResourceCheck):
    def __init__(self):
        name = "Ensure every security groups rule has a description"
        id = "CKV_AWS_23"
        supported_resource = ['AWS::EC2::SecurityGroup', 'AWS::EC2::SecurityGroupIngress', 'AWS::EC2::SecurityGroupEgress', 'AWS::RDS::DBSecurityGroup',
                              'AWS::ElastiCache::SecurityGroup', 'AWS::Redshift::ClusterSecurityGroup']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def scan_resource_conf(self, conf):
        """
            Looks for group description and description in security group rules :
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-ec2-security-group.html
        :param conf: aws_security_group configuration
        :return: <CheckResult>
        """

        if conf['Type'] == 'AWS::EC2::SecurityGroup':
            if 'Properties' in conf.keys():
                if 'GroupDescription' in conf['Properties'].keys():
                    if 'SecurityGroupIngress' in conf['Properties'].keys():
                        for rule in conf['Properties']['SecurityGroupIngress']:
                            if 'Description' not in rule.keys():
                                return CheckResult.FAILED
                    if 'SecurityGroupEgress' in conf['Properties'].keys():
                        for rule in conf['Properties']['SecurityGroupEgress']:
                            if 'Description' not in rule.keys():
                                return CheckResult.FAILED
                    return CheckResult.PASSED
                else:
                    return CheckResult.FAILED
        elif conf['Type'] == 'AWS::EC2::SecurityGroupIngress' or conf['Type'] == 'AWS::EC2::SecurityGroupEgress':
            if 'Properties' in conf.keys() and 'Description' in conf['Properties'].keys():
                return CheckResult.PASSED
        elif conf['Type'] == 'AWS::RDS::DBSecurityGroup':
            if 'Properties' in conf.keys() and 'GroupDescription' in conf['Properties'].keys():
                return CheckResult.PASSED
        elif conf['Type'] == 'AWS::ElastiCache::SecurityGroup' or conf['Type'] == 'AWS::Redshift::ClusterSecurityGroup':
            if 'Properties' in conf.keys() and 'Description' in conf['Properties'].keys():
                return CheckResult.PASSED

        return CheckResult.FAILED


check = SecurityGroupRuleDescription()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class DefaultNamespace(BaseK8Check):

    def __init__(self):
        # CIS-1.5 5.7.4
        name = "The default namespace should not be used"
        id = "CKV_K8S_21"
        supported_kind = ['Pod', 'Deployment', 'DaemonSet', 'StatefulSet', 'ReplicaSet', 'ReplicationController', 'Job', 'CronJob', 'Service', 'Secret', 'ServiceAccount', 'Role', 'RoleBinding', 'ConfigMap', 'Ingress']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return conf['kind'] + '.metadata.namespace'

    def scan_spec_conf(self, conf):
        if "metadata" in conf:
            if "namespace" in conf["metadata"]:
                if conf["metadata"]["namespace"] != "default":
                    return CheckResult.PASSED
            return CheckResult.FAILED
        return CheckResult.FAILED

check = DefaultNamespace()


from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class ReadOnlyFilesystem(BaseK8Check):

    def __init__(self):
        name = "Use read-only filesystem for containers where possible"
        id = "CKV_K8S_22"
        supported_kind = ['containers', 'initContainers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return conf['parent'] + '.securityContext.readOnlyRootFilesystem'

    def scan_spec_conf(self, conf):
        if "securityContext" in conf:
            if "readOnlyRootFilesystem" in conf["securityContext"]:
                if conf["securityContext"]["readOnlyRootFilesystem"]:
                    return CheckResult.PASSED
        return CheckResult.FAILED

check = ReadOnlyFilesystem()


from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class PrivilegedContainers(BaseK8Check):

    def __init__(self):
        # CIS-1.3 1.7.1
        # CIS-1.5 5.2.1
        name = "Container should not be privileged"
        id = "CKV_K8S_16"
        supported_kind = ['containers', 'initContainers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return conf['parent'] + '.securityContext.privileged'

    def scan_spec_conf(self, conf):
        if "securityContext" in conf:
            if "privileged" in conf["securityContext"]:
                if conf["securityContext"]["privileged"]:
                    return CheckResult.FAILED
        return CheckResult.PASSED

check = PrivilegedContainers()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class ContainerSecurityContext(BaseK8Check):

    def __init__(self):
        # CIS-1.5 5.7.3
        name = "Apply security context to your pods and containers"
        # Security context can be set at pod or container level.
        id = "CKV_K8S_30"
        supported_kind = ['containers', 'initContainers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return conf['parent'] + '.securityContext'

    def scan_spec_conf(self, conf):
        if "securityContext" in conf:
            if conf["securityContext"]:
                    return CheckResult.PASSED
        return CheckResult.FAILED

check = ContainerSecurityContext()



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class GoogleCloudSqlDatabaseRequireSsl(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all Cloud SQL database instance requires all incoming connections to use SSL"
        id = "CKV_GCP_6"
        supported_resources = ['google_sql_database_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for google_sql_database_instance which do not enforce SSL connections:
        :param conf: google_sql_database_instance configuration
        :return: <CheckResult>
        """
        if 'settings' in conf and 'ip_configuration' in conf['settings'][0]:
            if 'require_ssl' in conf['settings'][0]['ip_configuration'][0].keys():
                if conf['settings'][0]['ip_configuration'][0]['require_ssl'][0] == True:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = GoogleCloudSqlDatabaseRequireSsl()
